module doodle.fig.select_tool;

public {
    import doodle.dia.tool;
}

private {
    import doodle.tk.cairo;
}

final class SelectTool : Tool {
    this() {
        super("Select");
    }

    override bool handleButtonPress(scope IViewport viewport, in ButtonEvent event) {
        if (event.buttonName == ButtonName.LEFT) {
            _active = true;
            _anchorPoint = _currentPoint = event.pixelPoint;
            viewport.setCursor(Cursor.HAND);
            return true;
        }
        else {
            return false;
        }
    }

    override bool handleButtonRelease(scope IViewport viewport, in ButtonEvent event) {
        if (event.buttonName == ButtonName.LEFT && _active) {
            _active = false;
            viewport.damageScreen(growCentre(Rectangle(_anchorPoint, _currentPoint), LINE_WIDTH));
            viewport.setCursor(Cursor.DEFAULT);
            return true;
        }
        else {
            return false;
        }
    }

    override bool handleMotion(scope IViewport viewport, in MotionEvent event) {
        if (_active) {
            viewport.damageScreen(growCentre(Rectangle(_anchorPoint, _currentPoint), LINE_WIDTH));
            _currentPoint = event.pixelPoint;
            viewport.damageScreen(growCentre(Rectangle(_anchorPoint, _currentPoint), LINE_WIDTH));
        }

        return false;
    }

    override void draw(in Rectangle pixelDamage, scope Context pixelCr) const {
        if (_active) {
            /*
               pixelCr.save; {
               pixelCr.setSourceRgba(0.0, 0.0, 0.8, 0.3);
               rectangle(pixelCr, Rectangle(_currentPoint, _anchorPoint));
               pixelCr.fill();
               } pixelCr.restore();
             */

            pixelCr.save(); {
                double[] dashes = [ 4.0, 4.0 ];
                pixelCr.setDash(dashes, 0.0);
                pixelCr.setSourceRgba(0.0, 0.0, 0.5, 1.0);
                pixelCr.setLineWidth(LINE_WIDTH);
                rectangle(pixelCr, Rectangle(_currentPoint, _anchorPoint));
                pixelCr.stroke;
            } pixelCr.restore;
        }
    }

    private {
        bool _active;
        Point _currentPoint;
        Point _anchorPoint;      // Pixel
        static immutable double LINE_WIDTH = 1.0;
    }
}
